local t = 1.2

-- this function is run when we run out of commands to issue and enough time has passed to issue more commands
local function sequence()
  mem.combined = {} -- this is a table of orders to be executed by the other Luacontroller
  local new_storage = { [0] = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
  local timer = 0 -- this is how long this processing sequence is expected to take

  for value = 1, #mem.storage - 1 do -- only execute over 1 to 34, as 0 and 35 cannot be split
    local count = mem.storage[value]
    if count > 1 then
      -- calculate changes
      local new_count = math.floor(count / 2)
      local low_value = value - 1
      local high_value = value + 1
      local remainder = count % 2

      -- save the command that will execute this change
      table.insert(mem.combined, count - remainder) -- how many, minus one if odd number exists
      table.insert(mem.combined, value) -- which value we are operating on

      -- save the resulting changes
      new_storage[low_value] = new_storage[low_value] + new_count
      new_storage[high_value] = new_storage[high_value] + new_count
      new_storage[value] = new_storage[value] + remainder

      -- increment timer
      timer = timer + count * 2 * t
    end
  end

  mem.storage = new_storage
  mem.timer = timer
end

if event.type == "program" then
  -- previously worked by combining an array of how many and an array of their values
  --mem.counts = {6,2,5,8,6,4,4,2,2,2,4,5,2,3}
  --mem.values = {8,1,6,12,3,11,10,16,7,13,14,5,2,9}

  -- now they are ordered in pairs: count, then its value
  --mem.combined = {4,16,3,6,3,11,5,14,5,10,5,7,4,3,5,5,2,1,5,12,5,9}

  -- newest variant keeps a running storage value
  mem.storage = mem.storage or { [0] = 1, 13, 12, 25, 2, 15, 19, 28, 22, 1, 22, 24, 10, 10, 20, 2, 1, 3, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }

  -- NOPE: we wait for a signal to reprogram us!
  -- now we generate mem.combined from this, and the resulting storage
  --sequence()

  --interrupt(t) -- next interrupt will begin sending these commands

elseif event.type == "interrupt" then
  --count = table.remove(mem.counts, 1)
  --value = table.remove(mem.values, 1)

  local count = table.remove(mem.combined, 1)
  local value = table.remove(mem.combined, 1)

  -- if there are no more commands to send, time to calculate and prepare the next sequence of commands
  if not count then
    sequence()
    interrupt(mem.timer) -- we will begin sending orders only after a delay, so that existing orders finish executing
    return
  end

  --if count % 2 ~= 0 then count = count - 1 end -- automatically ignores odd numbers (legacy code, our sequencer handles this automatically!)

  digiline_send("request", {count, value})
  digiline_send("request", {count}) -- we send an extra invalid sequence to force the program to wait extra time

  interrupt(t)

elseif event.type == "digiline" then
  if event.channel == "abort" then
    mem.timer = 0
    mem.storage = {}
    mem.combined = {}
  elseif event.channel == "program" then
    mem.storage = event.msg
    sequence()
    interrupt(t)
  end
end